##
##   lts.conf dictionary
##
##   Copyright (C) 2007 Canonical Ltd.
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2, or (at your option)
##   any later version.
##
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.

"""
ltsconf_dict is a module to validate lts.conf values and to detect 
all possible values in a ltsp chroot

Copyright 2007, Canonical Ltd.
"""

import os
from subprocess import Popen,PIPE

__all__='keymaps','kbmodels','videodrivers','serproto','isboolean','validate'

valid = ['SOUND',
        'SOUND_DAEMON',
        'LOCALDEV',
        'DNS_SERVER',
        'SEARCH_DOMAIN',
        'CONSOLE_KEYMAP',
        'NBD_SWAP',
        'SWAP_SERVER',
        'NBD_PORT',
        'USE_LOCAL_SWAP',
        'PRINTER_0_DEVICE',
        'PRINTER_0_TYPE',
        'PRINTER_0_PORT',
        'PRINTER_0_WRITE_ONLY',
        'PRINTER_0_SPEED',
        'PRINTER_0_FLOWCTRL',
        'PRINTER_0_PARITY',
        'PRINTER_0_DATABITS',
        'PRINTER_0_OPTIONS',
        'USE_XFS',
        'XFS_SERVER',
        'XKBLAYOUT',
        'XKBMODEL',
        'XKBRULES',
        'XKBOPTIONS',
        'XKBVARIANT',
        'NETWORK_COMPRESSION',
        'LDM_REMOTECMD',
        'RCFILE_nn',
        'SCREEN_nn',
        'SERVER',
        'SYSLOG_HOST',
        'MODULE_nn',
        'XF86CONFIG_FILE',
        'XSERVER',
        'X_HORZSYNC',
        'X_VERTREFRESH',
        'X_COLOR_DEPTH',
        'X_MOUSE_DEVICE',
        'X_MOUSE_PROTOCOL',
        'X_MOUSE_EMULATE3BTN',
        'TELNET_HOST',
        'XDM_SERVER']

boolean=['SOUND',
        'LOCALDEV',
        'NBD_SWAP',
        'USE_LOCAL_SWAP',
        'USE_XFS',
        'NETWORK_COMPRESSION',
        'X_MOUSE_EMULATE3BTN']

def keymaps(root):
    """
    returns an array of available X keymaps in root

    root: a full ltsp client chroot path, i.e. /opt/ltsp/i386
    """
    res=[]
    for keymap in os.listdir(root+'/etc/X11/xkb/keymap/'):
        if keymap == 'README':
            continue
        try:
            keymap_file = file(os.path.join(root, 'etc/X11/xkb/keymap',keymap))
        except:
            continue
        else:
            for line in keymap_file:
                if line.find('xkb_keymap') >= 0:
                    res.append(line.split('"')[1])
    return unique(res)

def kbmodels(root):
    """
    returns an array of known X keyboard models in root
    
    root: a full ltsp client chroot path, i.e. /opt/ltsp/i386
    """
    ret = []
    for kbmodel in os.listdir(root+'/etc/X11/xkb/geometry/'):
        if kbmodel == 'README':
            continue
        try:
            kbmodel_file = file(os.path.join(root, 'etc/X11/xkb/geometry',kbmodel))
        except:
            continue
        else:
            for line in kbmodel_file:
                if line.find('xkb_geometry') >= 0:
                    ret.append(line.split('"')[1])
    return ret

def videodrivers(root):
    """
    returns an array of known X drivers in root
    
    root: a full ltsp client chroot path, i.e. /opt/ltsp/i386
    """
    ret=[]
    cardlist = os.listdir(os.path.join(root, 'usr/lib/xorg/modules/drivers'))
    for item in cardlist:
        if item.find('.so') >= 0:
                ret.append(item.split('.')[0].split('_drv')[0])
    print ret

def serproto(root):
    """
    returns an array of known serial mouse protocol options in root
    
    root: a full ltsp client chroot path, i.e. /opt/ltsp/i386
    """
    ret = []
    modelist = Popen(root+'/usr/sbin/inputattach --help', shell=True, bufsize=1024, stdout=PIPE).stdout
    for line in modelist:
        if line.lstrip().startswith('--'):
            ret.append(line.lstrip('  --').split()[0])
    return ret

def unique(s):
             n = len(s)
             if n == 0:
                     return []
             u = {}
             try:
                     for x in s:
                             u[x] = 1
             except TypeError:
                     del u  # move on to the next method
             else:
                     return u.keys()
             
             try:
                     t = list(s)
                     t.sort()
             except TypeError:
                     del t  # move on to the next method
             else:
                     assert n > 0
                     last = t[0]
                     lasti = i = 1
                     while i < n:
                             if t[i] != last:
                                     t[lasti] = last = t[i]
                                     lasti += 1
                             i += 1
                     return t[:lasti]
             
             # Brute force is all that's left.
             u = []
             for x in s:
                     if x not in u:
                             u.append(x)
             return u

def isboolean(var):
    """
    returns True if var is a valid variable for lts.conf

    var: variable name (i.e. 'SOUND')
    """
    if var in boolean:
        return True
    return False

def validate(var):
    """
    returns true if var is a valid variable for lts.conf

    var: variable name (i.e. 'SOUND')
    """
    if var in valid:
        return True
    return False



##
##   lts.conf parser
##
##   Copyright (C) 2007 Canonical Ltd.
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2, or (at your option)
##   any later version.
##
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.

"""
ltsconf reads and writes the lts.conf file for ltsp servers, 
does an automatic backup and has a rollback function to the 
backed up config.

Copyright 2007, Canonical Ltd.
"""

import os
import os.path

def read(path):
    """
    returns an array of triplets (index, key, value) and an index list
    read from lts.conf in path.
    example: data_array,idx=ltsconf_read(./lts.conf)
             print data_array
             [('default', 'SOUND', 'True'),('default','LOCALDEV','True'), ('...]
             print idx
             ['default','00:11:25:84:CE:BA']

    path: full path to lts.conf file
    """

    idx = []
    data_array = []
    if os.path.isfile(path):
        ltsconf = open(path)
        ltsp_data = ltsconf.read().split('\n')
    
        for line in ltsp_data:
            line = line.strip()
            if not line.startswith('#'):
                if line.startswith('['):
                    index = line.strip('[]')
                    idx.append(index)
                else:
                    if line:
                        key = line.split('=')[0]
                        value = line.split('=')[1]
                        data_array.append((index,key,value))
        return data_array, idx

def write(path, data_array, idx):
    """
    writes an lts.conf in path, containing groups listed in idx and values in 
    those groups from data_array. by default path/.lts.conf.bak is created before 
    writing the new file.

    path: full path to an lts.conf file
    data_array: array of triplets [(index,key,value), (index, key2, value2), ...]
    idx: list of indexes [index1, index2, ... ]
    """
    if os.path.isfile(path):
        os.rename(path, path.rstrip('lts.conf')+'.lts.conf.bak')
    ltsconf = open(path, 'w')
    ltsconf.write('###########################################\n')
    ltsconf.write('# AUTOGENERATED FILE, DO NOT EDIT !       #\n')
    ltsconf.write('# Use the thinclient-configurator instead #\n')
    ltsconf.write('# A backup of the last configuration can  #\n')
    ltsconf.write('# be found hidden in the same path, named #\n')
    ltsconf.write('# .lts.conf.bak                           #\n')
    ltsconf.write('###########################################\n')
    for index in idx:
        ltsconf.write('\n['+index+']\n')
        for line in data_array:
            if line[0] == index:
                ltsconf.write('\t'+line[1]+'='+line[2]+'\n')
    return True

def rollback(path):
    """
    rolls back the config from a backed up ltsp configuration if path/.lts.conf.bak
    exists.

    path: full path to an lts.conf file
    """
    if os.path.isfile(path):
        os.rename(path.rstrip('lts.conf')+'.lts.conf.bak', path)
        return True
    return False



##
##   /etc/ltsp/dhcpd,conf parser
##
##   Copyright (C) 2007 Canonical Ltd.
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2, or (at your option)
##   any later version.
##
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.

def read():
        list = []
	try:
		file = open('/etc/ltsp/dhcpd.conf', 'r')
	except:
                print 'can\'t open dhcp config'
                return False
	for line in file.read().split('\n'):
            line = line.strip().lstrip('option ')
            items = ['range','domain-name','domain-name-servers','broadcast-address','routers','subnet-mask','filename','root-path']
            for item in items:
                if line.startswith(item+' '):
                    if item == 'range':
                        range_from = line.strip('range').strip(';').split()[0]
                        range_to = line.strip('range').strip(';').split()[1]
                        list.append(('range_from', range_from))
                        list.append(('range_to',range_to))
                    else:
                        list.append((item, line.strip(item).strip(';').split()[0].strip('"')))
        return list



##
##   ltsp services administration
##
##   Copyright (C) 2007 Canonical Ltd.
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2, or (at your option)
##   any later version.
##
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.

"""
manage additional service configs in /etc/ltsp/ (i.e. syslog configuration)
"""



#!/usr/bin/python
# -*- coding: koi8-r -*-
from distutils.core import setup,sys
import os

if sys.version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None

# Set proper release version in source code also!!!
setup(name='ltsp',
      version='0.1',
      author='Oliver Grawert',
      author_email='ogra@ubuntu.com',
      url='http://launchpad.net/python-ltsp/',
      description='functions to access ltsp servers',
      long_description="""This library provides functionality for writing ltsp 
admin interfaces.

It is distributed under the terms of GPL.""",
      download_url='http://launchpad.net/python-ltsp/',
      packages=['ltsp'],
      license="GPL",
      platforms="All",
      keywords=['ltsp','thinclient'],
      classifiers = [
          'Topic :: System :: Systems Administration',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Natural Language :: English',
          'Development Status :: 1 - Alpha',
          'Intended Audience :: Developers',
        ],
     )



##
##   LTSP server status
##
##   Copyright (C) 2007 Canonical Ltd.
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2, or (at your option)
##   any later version.
##
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.

"""
status offers simple functions to check the different 
server processes used by ltsp servers.

Copyright 2007, Canonical Ltd.
"""

import os

def check_server(binary_name):
    """
    checks if binary_name is currently running and returns True/False

    binary_name: name of a server bianry to look for in the processlist (i.e. nfsd)
    """
    status=os.popen('ps -C '+binary_name+' --no-headers')
    for line in status:
        if line:
            return True
    return False

def check_inetd(service):
    """
    checks if service is enabled in /etc/inetd.conf and checks if inetd is
    running. returns True/False
        
    service: service name (i.e. tftp) or a portnumber
    """
    status=os.popen('grep ^'+service+' /etc/inetd.conf')
    if status.read():
        return check_server('inetd')
    return False

def tftp():
    """
    returns True if tftp service is ok
    """
    return check_inetd('tftp')

def ldminfo():
    """
    returns True if ldminfo service is ok
    """
    return check_inetd('9571')

def nbdswap():
    """
    returns True if nbdswap service is ok
    """
    return check_inetd('9572')

def dhcp():
    """
    returns True if dhcp service is ok
    """
    return check_server('dhcpd3')

def nfs():
    """
    returns True if nfs service is ok
    """
    return check_server('nfsd')
